@@ -58,10 +58,8 @@ using namespace nghttp2;
 namespace shrpx {
 
 namespace {
-constexpr StringRef SEVERITY_STR[] = {
-    StringRef::from_lit("INFO"), StringRef::from_lit("NOTICE"),
-    StringRef::from_lit("WARN"), StringRef::from_lit("ERROR"),
-    StringRef::from_lit("FATAL")};
+constexpr StringRef SEVERITY_STR[] = {"INFO"_sr, "NOTICE"_sr, "WARN"_sr,
+                                      "ERROR"_sr, "FATAL"_sr};
 } // namespace
 
 namespace {
@@ -90,7 +88,7 @@ pthread_once_t lckey_once = PTHREAD_ONCE_INIT;
 } // namespace
 
 namespace {
-void make_key() { pthread_key_create(&lckey, NULL); }
+void make_key() { pthread_key_create(&lckey, nullptr); }
 } // namespace
 
 LogBuffer *get_logbuf() {
@@ -116,11 +114,10 @@ int Log::severity_thres_ = NOTICE;
 
 void Log::set_severity_level(int severity) { severity_thres_ = severity; }
 
-int Log::set_severity_level_by_name(const StringRef &name) {
+int Log::get_severity_level_by_name(const StringRef &name) {
   for (size_t i = 0, max = array_size(SEVERITY_STR); i < max; ++i) {
     if (name == SEVERITY_STR[i]) {
-      severity_thres_ = i;
-      return 0;
+      return i;
     }
   }
   return -1;
@@ -174,11 +171,10 @@ Log::~Log() {
   if (errorconf.syslog) {
     if (severity_ == NOTICE) {
       syslog(severity_to_syslog_level(severity_), "[%s] %.*s",
-             SEVERITY_STR[severity_].c_str(), static_cast<int>(rleft()),
-             begin_);
+             SEVERITY_STR[severity_].data(), static_cast<int>(rleft()), begin_);
     } else {
       syslog(severity_to_syslog_level(severity_), "[%s] %.*s (%s:%d)",
-             SEVERITY_STR[severity_].c_str(), static_cast<int>(rleft()), begin_,
+             SEVERITY_STR[severity_].data(), static_cast<int>(rleft()), begin_,
              filename_, linenum_);
     }
 
@@ -190,13 +186,13 @@ Log::~Log() {
 
   lgconf->update_tstamp_millis(std::chrono::system_clock::now());
 
-  // Error log format: <datetime> <master-pid> <current-pid>
+  // Error log format: <datetime> <main-pid> <current-pid>
   // <thread-id> <level> (<filename>:<line>) <msg>
   rv = snprintf(buf, sizeof(buf), "%s %d %d %s %s%s%s (%s:%d) %.*s\n",
-                lgconf->tstamp->time_iso8601.c_str(), config->pid, lgconf->pid,
+                lgconf->tstamp->time_iso8601.data(), config->pid, lgconf->pid,
                 lgconf->thread_id.c_str(), tty ? SEVERITY_COLOR[severity_] : "",
-                SEVERITY_STR[severity_].c_str(), tty ? "\033[0m" : "",
-                filename_, linenum_, static_cast<int>(rleft()), begin_);
+                SEVERITY_STR[severity_].data(), tty ? "\033[0m" : "", filename_,
+                linenum_, static_cast<int>(rleft()), begin_);
 
   if (rv < 0) {
     return;
@@ -252,7 +248,7 @@ Log &Log::operator<<(long long n) {
     return *this;
   }
   *last_++ = '-';
-  *last_ += nlen;
+  last_ += nlen;
   update_full();
 
   auto p = last_ - 1;
@@ -381,7 +377,7 @@ namespace {
 template <typename OutputIterator>
 std::pair<OutputIterator, OutputIterator>
 copy(const StringRef &src, OutputIterator d_first, OutputIterator d_last) {
-  return copy(src.c_str(), src.size(), d_first, d_last);
+  return copy(src.data(), src.size(), d_first, d_last);
 }
 } // namespace
 
@@ -532,7 +528,7 @@ template <typename OutputIterator>
 std::pair<OutputIterator, OutputIterator> copy_escape(const StringRef &src,
                                                       OutputIterator d_first,
                                                       OutputIterator d_last) {
-  return copy_escape(src.c_str(), src.size(), d_first, d_last);
+  return copy_escape(src.data(), src.size(), d_first, d_last);
 }
 } // namespace
 
@@ -556,7 +552,7 @@ StringRef construct_absolute_request_uri(BlockAllocator &balloc,
   }
 
   auto iov = make_byte_ref(balloc, len + 1);
-  auto p = iov.base;
+  auto p = std::begin(iov);
 
   if (req.scheme.empty()) {
     // We may have to log the request which lacks scheme (e.g.,
@@ -570,7 +566,7 @@ StringRef construct_absolute_request_uri(BlockAllocator &balloc,
   p = std::copy(std::begin(req.path), std::end(req.path), p);
   *p = '\0';
 
-  return StringRef{iov.base, p};
+  return StringRef{std::span{std::begin(iov), p}};
 }
 } // namespace
 
@@ -594,15 +590,18 @@ void upstream_accesslog(const std::vector<LogFragment> &lfv,
   auto &balloc = downstream->get_block_allocator();
 
   auto downstream_addr = downstream->get_addr();
-  auto method = http2::to_method_string(req.method);
-  auto path = req.method == HTTP_CONNECT
-                  ? req.authority
-                  : config->http2_proxy
-                        ? construct_absolute_request_uri(balloc, req)
-                        : req.path.empty() ? req.method == HTTP_OPTIONS
-                                                 ? StringRef::from_lit("*")
-                                                 : StringRef::from_lit("-")
-                                           : req.path;
+  auto method =
+      req.method == -1 ? "<unknown>"_sr : http2::to_method_string(req.method);
+  auto path = req.method == HTTP_CONNECT ? req.authority
+              : config->http2_proxy
+                  ? construct_absolute_request_uri(balloc, req)
+              : req.path.empty() ? req.method == HTTP_OPTIONS ? "*"_sr : "-"_sr
+                                 : req.path;
+  auto path_without_query =
+      req.method == HTTP_CONNECT
+          ? path
+          : StringRef{std::begin(path),
+                      std::find(std::begin(path), std::end(path), '?')};
 
   auto p = std::begin(buf);
   auto last = std::end(buf) - 2;
@@ -632,6 +631,23 @@ void upstream_accesslog(const std::vector<LogFragment> &lfv,
         std::tie(p, last) = copy(req.http_minor, p, last);
       }
       break;
+    case LogFragmentType::METHOD:
+      std::tie(p, last) = copy(method, p, last);
+      break;
+    case LogFragmentType::PATH:
+      std::tie(p, last) = copy_escape(path, p, last);
+      break;
+    case LogFragmentType::PATH_WITHOUT_QUERY:
+      std::tie(p, last) = copy_escape(path_without_query, p, last);
+      break;
+    case LogFragmentType::PROTOCOL_VERSION:
+      std::tie(p, last) = copy_l("HTTP/", p, last);
+      std::tie(p, last) = copy(req.http_major, p, last);
+      if (req.http_major < 2) {
+        std::tie(p, last) = copy('.', p, last);
+        std::tie(p, last) = copy(req.http_minor, p, last);
+      }
+      break;
     case LogFragmentType::STATUS:
       std::tie(p, last) = copy(resp.http_status, p, last);
       break;
@@ -735,7 +751,11 @@ void upstream_accesslog(const std::vector<LogFragment> &lfv,
         std::tie(p, last) = copy('-', p, last);
         break;
       }
+#if OPENSSL_3_0_0_API
+      auto x = SSL_get0_peer_certificate(lgsp.ssl);
+#else  // !OPENSSL_3_0_0_API
       auto x = SSL_get_peer_certificate(lgsp.ssl);
+#endif // !OPENSSL_3_0_0_API
       if (!x) {
         std::tie(p, last) = copy('-', p, last);
         break;
@@ -746,7 +766,9 @@ void upstream_accesslog(const std::vector<LogFragment> &lfv,
           lf.type == LogFragmentType::TLS_CLIENT_FINGERPRINT_SHA256
               ? EVP_sha256()
               : EVP_sha1());
+#if !OPENSSL_3_0_0_API
       X509_free(x);
+#endif // !OPENSSL_3_0_0_API
       if (len <= 0) {
         std::tie(p, last) = copy('-', p, last);
         break;
@@ -760,7 +782,11 @@ void upstream_accesslog(const std::vector<LogFragment> &lfv,
         std::tie(p, last) = copy('-', p, last);
         break;
       }
+#if OPENSSL_3_0_0_API
+      auto x = SSL_get0_peer_certificate(lgsp.ssl);
+#else  // !OPENSSL_3_0_0_API
       auto x = SSL_get_peer_certificate(lgsp.ssl);
+#endif // !OPENSSL_3_0_0_API
       if (!x) {
         std::tie(p, last) = copy('-', p, last);
         break;
@@ -768,7 +794,9 @@ void upstream_accesslog(const std::vector<LogFragment> &lfv,
       auto name = lf.type == LogFragmentType::TLS_CLIENT_ISSUER_NAME
                       ? tls::get_x509_issuer_name(balloc, x)
                       : tls::get_x509_subject_name(balloc, x);
+#if !OPENSSL_3_0_0_API
       X509_free(x);
+#endif // !OPENSSL_3_0_0_API
       if (name.empty()) {
         std::tie(p, last) = copy('-', p, last);
         break;
@@ -781,13 +809,19 @@ void upstream_accesslog(const std::vector<LogFragment> &lfv,
         std::tie(p, last) = copy('-', p, last);
         break;
       }
+#if OPENSSL_3_0_0_API
+      auto x = SSL_get0_peer_certificate(lgsp.ssl);
+#else  // !OPENSSL_3_0_0_API
       auto x = SSL_get_peer_certificate(lgsp.ssl);
+#endif // !OPENSSL_3_0_0_API
       if (!x) {
         std::tie(p, last) = copy('-', p, last);
         break;
       }
       auto sn = tls::get_x509_serial(balloc, x);
+#if !OPENSSL_3_0_0_API
       X509_free(x);
+#endif // !OPENSSL_3_0_0_API
       if (sn.empty()) {
         std::tie(p, last) = copy('-', p, last);
         break;
@@ -842,7 +876,7 @@ int reopen_log_files(const LoggingConfig &loggingconf) {
   auto &errorconf = loggingconf.error;
 
   if (!accessconf.syslog && !accessconf.file.empty()) {
-    new_accesslog_fd = open_log_file(accessconf.file.c_str());
+    new_accesslog_fd = open_log_file(accessconf.file.data());
 
     if (new_accesslog_fd == -1) {
       LOG(ERROR) << "Failed to open accesslog file " << accessconf.file;
@@ -851,7 +885,7 @@ int reopen_log_files(const LoggingConfig &loggingconf) {
   }
 
   if (!errorconf.syslog && !errorconf.file.empty()) {
-    new_errorlog_fd = open_log_file(errorconf.file.c_str());
+    new_errorlog_fd = open_log_file(errorconf.file.data());
 
     if (new_errorlog_fd == -1) {
       if (lgconf->errorlog_fd != -1) {
@@ -944,7 +978,7 @@ int open_log_file(const char *path) {
       strcmp(path, "/proc/self/fd/2") == 0) {
     return STDERR_COPY;
   }
-#if defined O_CLOEXEC
+#ifdef O_CLOEXEC
 
   auto fd = open(path, O_WRONLY | O_APPEND | O_CREAT | O_CLOEXEC,
                  S_IRUSR | S_IWUSR | S_IRGRP);
